﻿

Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Customization
Imports oApp = Autodesk.AutoCAD.ApplicationServices.Application


Public Class Class1
    '<DllImport("acad.exe", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl, EntryPoint = "acedCmd")>
    'private static extern int acedCmd(System.IntPtr vlist);
    Private Declare Function acedCmd Lib "acad.exe" Alias "acedCmd" (ByVal vlist As System.IntPtr) As Integer



    <CommandMethod("addPartial")> _
    Public Sub AddPartial()
        Dim ed As Editor = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor
        Try
            'retrieve the location of, and open the ACAD Main CUI File
            Dim mainCuiFile As String = oApp.GetSystemVariable("MENUNAME")
            mainCuiFile += ".cuix"

            Dim cs As CustomizationSection = New CustomizationSection(mainCuiFile)
            Dim partials As PartialCuiFileCollection = cs.PartialCuiFiles

            Dim csPartial As CustomizationSection
            Dim strPartialCuiFile As String
            strPartialCuiFile = "c:\temp\PartialWithTab.cuix"
            Dim bPartialLoaded As Boolean = False

            'Check if the partial is already loaded
            If (Not partials.Contains(strPartialCuiFile)) Then
                'Make sure the partial CUI file exists
                If (Not System.IO.File.Exists(strPartialCuiFile)) Then
                    ed.WriteMessage("Partial CUI file does not exist. exiting")
                    Return
                End If
                'The partial cuix file exists but it is not loaded so add
                ' it to the main cuix
                csPartial = New CustomizationSection(strPartialCuiFile)
                cs.PartialCuiFiles.Add(strPartialCuiFile)
            Else
                'The partial cuix is already loaded in the main cuix
                csPartial = New CustomizationSection(strPartialCuiFile)
                bPartialLoaded = True
                'cs.Save()
            End If

            Dim wsCol As WorkspaceCollection = cs.Workspaces
            If wsCol.Count <= 0 Then
                ed.WriteMessage("Failed to Get the WorkspaceCollection")
                Return
            End If

            Dim wrkSpaceRibbonRoot As WSRibbonRoot = Nothing
            Dim strWrkSpaceName As String = "2D Drafting & Annotation"
            Dim i As Integer
            For i = 0 To wsCol.Count - 1
                'If the name of the Workspace is "2d Drafting & Annotation"
                ' then get the WSRibbonRoot
                If wsCol(i).Name = strWrkSpaceName Then
                    wrkSpaceRibbonRoot = wsCol(i).WorkspaceRibbonRoot
                    Exit For
                End If
            Next i

            If wrkSpaceRibbonRoot Is Nothing Then
                ed.WriteMessage("2D Drafting & Annotation WSRibbonRoot Not found")
                Return
            End If

            'Need to change this to an elementID of a tab in your partial cuix
            Dim strTabId As String = "RBNU_0001"

            Dim wsRibbonTabCollection As WorkspaceRibbonTabCollection = Nothing
            'Get the Tabcollection in the 2d Drafting & Annotation work space
            wsRibbonTabCollection = wrkSpaceRibbonRoot.WorkspaceTabs
            'See if Tab already exists in the 2D Drafting & Annotation Workspace
            Dim wsRibTabSrcRef As Autodesk.AutoCAD.Customization.WSRibbonTabSourceReference
            For Each wsRibTabSrcRef In wsRibbonTabCollection
                'If wsRibTabSrcRef2.ElementID = strTabId Then
                If wsRibTabSrcRef.TabId = strTabId Then
                    ed.WriteMessage("Tab already exists in 2D Drafting & Annotation")
                    Return
                End If
            Next wsRibTabSrcRef

            'Get the Tab in the partial
            'Dim wsRibTabSrcRef As Autodesk.AutoCAD.Customization.WSRibbonTabSourceReference
            wsRibTabSrcRef = getTabInPartial(csPartial, strTabId)

            If wsRibTabSrcRef Is Nothing Then

                ed.WriteMessage("Failed to create WSRibbonTabSourceReference")
                Return
            End If

            If (wsRibbonTabCollection.Count <= 0) Then
                ed.WriteMessage("2D Drafting & Annotation does not currently contain any Tabs")
            End If

            'Get a count of the Ribbons in the ribbon collection in Acme Workspace
            Dim intialTabCount As Integer = wsRibbonTabCollection.Count

            ' Make the parent of the WSRibbonTabSourceReference the 2s Drafting & Annotation Workspace RibbonRoot
            wsRibTabSrcRef.SetParent(wrkSpaceRibbonRoot)

            'Add the WSRibbonTabSourceReference to the Acme Workspace Ribbon Tab collection
            wsRibbonTabCollection.Add(wsRibTabSrcRef)

            'Check the count of the Ribbon Tab collection verifies the new Tab was added 
            If wsRibbonTabCollection.Count <> intialTabCount + 1 Then

                ed.WriteMessage("Failed To add WSRibbonTabSourceReference to 2d Drafting & Annotation Workspace")
                Return
            End If

            cs.Save()

            Dim flName As String = cs.CUIFileBaseName
            Autodesk.AutoCAD.ApplicationServices.Application.SetSystemVariable("FILEDIA", 0)
            Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.SendStringToExecute("cuiunload " + flName + " ", False, False, False)
            Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.SendStringToExecute("cuiload " + flName + " ", False, False, False)
            'Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.SendStringToExecute("cuiload " + flName + " filedia 1 ", False, False, False)

            'Load the partial, without this need to run the cui dialog and switch
            'workspaces to get the Tab to display, even though WSCURRENT is run
            'and the 2d Drafting & Annotation is set current
            'If you do not want the Tab added to every workspace
            ' make the setting for behavior Merge Tab. The property for this is
            'RibbonTabSource.WorkspaceBehavior = TabWorkspaceBehavior.MergeTabOnly
            '(the automatic adding to workspaces does not occur with the enterprise cuix.
            Dim flPartialName As String = UCase(csPartial.CUIFileBaseName)
            Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.SendStringToExecute("cuiunload " + flPartialName + " ", False, False, True)
            Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.SendStringToExecute("cuiload " + strPartialCuiFile + " filedia 1 ", False, False, False)

            'Make the setting in the workspace current, using the WSCURRENT command
            Dim strLISPCmd As String = "(command " + """WSCURRENT""" + """2D Drafting & Annotation"") "
            Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.SendStringToExecute(strLISPCmd, False, False, True)

        Catch ex As System.Exception
            ed.WriteMessage(ex.ToString())

        Finally
        End Try
    End Sub
    Public Function getTabInPartial(ByVal csPartial As CustomizationSection, ByVal strTabId As String) As WSRibbonTabSourceReference
        Dim ed As Editor = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor

        Try
            Dim ribTabSrc As RibbonTabSource = Nothing
            Dim ribbonRoot As RibbonRoot = Nothing
            Dim wsRibTabSrcRef As WSRibbonTabSourceReference = Nothing

            ribbonRoot = csPartial.MenuGroup.RibbonRoot
            If ribbonRoot Is Nothing Then
                ed.WriteMessage("RibbonRoot Not found in partial")
                Return Nothing
            End If

            'Find the the Tab 
            ribTabSrc = ribbonRoot.FindTab(strTabId)

            ' Make sure the Ribbon Tab is found
            If ribTabSrc Is Nothing Then
                ed.WriteMessage("Ribbon Tab Not found\n")
                Return Nothing
            End If

            'Create the WSRibbonTabSourceReference using the RibbonTabSource
            wsRibTabSrcRef = WSRibbonTabSourceReference.Create(ribTabSrc)

            'Return the WSRibbonTabSourceReference
            Return wsRibTabSrcRef
        Catch ex As System.Exception
            ed.WriteMessage(ex.ToString())
            Return Nothing
        End Try
    End Function
    'This command is an exercize trying to remove tabs. 
    'If you do not want the tabs added to every workspace then just
    'make the setting for Tab behavior in the partial cuix Merge Only
    'this can be done in the API with this property
    'RibbonTabSource.WorkspaceBehavior = TabWorkspaceBehavior.MergeTabOnly
    '<CommandMethod("AddAndRemoveTabs")> _
    'Public Sub RemoveTab()
    '    Dim ed As Editor = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor
    '    Try
    '        'retrieve the location of, and open the ACAD Main CUI File
    '        Dim mainCuiFile As String = oApp.GetSystemVariable("MENUNAME")
    '        mainCuiFile += ".cuix"

    '        Dim cs As CustomizationSection = New CustomizationSection(mainCuiFile)
    '        Dim partials As PartialCuiFileCollection = cs.PartialCuiFiles

    '        Dim csPartial As CustomizationSection
    '        Dim strPartialCuiFile As String
    '        strPartialCuiFile = "c:\temp\PartialWithTab.cuix"
    '        Dim bPartialLoaded As Boolean = False

    '        'Check if the partial is already loaded
    '        If (Not partials.Contains(strPartialCuiFile)) Then
    '            'Make sure the partial CUI file exists
    '            If (Not System.IO.File.Exists(strPartialCuiFile)) Then
    '                ed.WriteMessage("Partial CUI file does not exist. exiting")
    '                Return
    '            End If
    '            'The partial cuix file exists but it is not loaded so add
    '            ' it to the main cuix
    '            csPartial = New CustomizationSection(strPartialCuiFile)

    '            cs.PartialCuiFiles.Add(strPartialCuiFile)
    '            cs.addPartialCUI(csPartial)
    '            cs.Save()

    '            'CUIUNLOAD main cuix
    '            Dim cuiFileName As String = cs.CUIFileBaseName
    '            Dim rb As ResultBuffer = New ResultBuffer()
    '            rb.Add(New TypedValue(5005, "_CUIUNLOAD"))
    '            rb.Add(New TypedValue(5005, cuiFileName))
    '            acedCmd(rb.UnmanagedObject)

    '            'CUILOAD Main cuix
    '            rb = New ResultBuffer()
    '            rb.Add(New TypedValue(5005, "_CUILOAD"))
    '            rb.Add(New TypedValue(5005, cuiFileName))
    '            acedCmd(rb.UnmanagedObject)

    '            'CUILOAD partial cuix
    '            'Dim cuiPartialFileName As String = strPartialCuiFile 'cs.CUIFileBaseName
    '            rb = New ResultBuffer()
    '            rb.Add(New TypedValue(5005, "_CUILOAD"))
    '            rb.Add(New TypedValue(5005, strPartialCuiFile))
    '            acedCmd(rb.UnmanagedObject)

    '            'cs.PartialCuiFiles.Add(strPartialCuiFile)
    '            'cs.addPartialCUI(csPartial)
    '            'cs.Save()
    '        Else
    '            'The partial cuix is already loaded in the main cuix
    '            csPartial = New CustomizationSection(strPartialCuiFile)
    '            bPartialLoaded = True
    '        End If

    '        Dim cs2 As CustomizationSection = New CustomizationSection(mainCuiFile)


    '        'Dim wsCol As WorkspaceCollection = cs.Workspaces
    '        Dim wsCol As WorkspaceCollection = cs2.Workspaces
    '        If wsCol.Count <= 0 Then
    '            ed.WriteMessage("Failed to Get the WorkspaceCollection")
    '            Return
    '        End If

    '        Dim wrkSpaceRibbonRoot As WSRibbonRoot = Nothing
    '        Dim strWrkSpaceName As String = "2D Drafting & Annotation"

    '        Dim i As Integer
    '        Dim wsRibTabSrcRef As Autodesk.AutoCAD.Customization.WSRibbonTabSourceReference
    '        Dim wsRibbonTabCollection As WorkspaceRibbonTabCollection = Nothing

    '        'Need to change this to an elementID of a tab in your partial cuix
    '        Dim strTabId As String = "RBNU_0001"

    '        For i = 0 To wsCol.Count - 1
    '            'do not want to remove the tab from 2D Drafting 
    '            If wsCol(i).Name.ToString() <> strWrkSpaceName Then

    '                wrkSpaceRibbonRoot = wsCol(i).WorkspaceRibbonRoot
    '                wsRibbonTabCollection = wrkSpaceRibbonRoot.WorkspaceTabs
    '                For Each wsRibTabSrcRef In wsRibbonTabCollection
    '                    If wsRibTabSrcRef.TabId = strTabId Then
    '                        ed.WriteMessage("Removing Tab from " + wsCol(i).Name.ToString())
    '                        wrkSpaceRibbonRoot.WorkspaceTabs.Remove(wsRibTabSrcRef)
    '                        Exit For
    '                    End If
    '                Next wsRibTabSrcRef
    '            End If

    '        Next i
    '        'cs.Save()
    '        cs2.Save()

    '        'Dim flName As String = cs.CUIFileBaseName
    '        Dim flName As String = cs2.CUIFileBaseName
    '        Autodesk.AutoCAD.ApplicationServices.Application.SetSystemVariable("FILEDIA", 0)
    '        Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.SendStringToExecute("cuiunload " + flName + " ", False, False, False)
    '        'Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.SendStringToExecute("cuiload " + flName + " ", False, False, False)
    '        Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.SendStringToExecute("cuiload " + flName + " filedia 1 ", False, False, False)

    '        'Make the setting in the workspace current, using the WSCURRENT command
    '        Dim strLISPCmd As String = "(command " + """WSCURRENT""" + """2D Drafting & Annotation"") "
    '        Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.SendStringToExecute(strLISPCmd, False, False, True)


    '    Catch ex As System.Exception
    '        ed.WriteMessage(ex.ToString())

    '    End Try
    'End Sub
End Class
